__all__ = ['Loader', 'NullLoader']

import warnings


class Loader:
    model = None
    serializer_class = None

    def __init__(self, source=None):
        self.source = source

    def load(self):
        if not self.source or not self.serializer_class:
            return

        for raw_data in self.source:
            data = self.parse(raw_data)
            serializer = self.serializer_class(data=data)
            if not serializer.is_valid():
                continue
                # TODO: warnings.warn()
            serializer.save()

    def parse(self, raw_data):
        return raw_data


class NullLoader(Loader):
    def load(self):
        return


__all__ = ['RegistryManager', 'register']

from django.contrib.contenttypes.models import ContentType
from django.db.models.fields.related import ForeignObject
from .loader import NullLoader


class Singleton(object):
    """A do-nothing class.
    From A. Martelli et al. Python Cookbook. (O'Reilly)
    Thanks to Juergen Hermann."""
    def __new__(cls, *args, **kwargs):
        if '_inst' not in vars(cls):
            cls._inst = super(Singleton, cls).__new__(cls, *args, **kwargs)
        return cls._inst


class RegistryManager(Singleton):
    DEFAULT_CATEGORY = 'default'
    _pool = {}

    def register(self, loader_class, category=DEFAULT_CATEGORY, source=None):
        key = (self._get_model_path(loader_class.model), category)
        self._pool[key] = loader_class(source=source)

    def _get_model_path(self, model):
        content_type = ContentType.objects.get_for_model(model)
        return '{}.{}'.format(content_type.app_label, content_type.model)

    def get_loader(self, model, category=DEFAULT_CATEGORY):
        key = (self._get_model_path(model), category)
        loader = self._pool.get(key, None)
        if loader is None:
            return NullLoader()
        else:
            return loader

    def clear(self):
        self._pool.clear()


def register(loader_class, category=RegistryManager.DEFAULT_CATEGORY, source=None):
    manager = RegistryManager()
    manager.register(loader_class, category=category, source=source)


def load_data(model, category=RegistryManager.DEFAULT_CATEGORY, with_deps=False):
    loader = RegistryManager().get_loader(model, category=category)
    if with_deps and loader.model is not None:
        for field in loader.model._meta.get_fields(include_hidden=True):
            if isinstance(field, ForeignObject):
                dep_model = field.related_model
                load_data(dep_model, category=category, with_deps=True)

    loader.load()



__all__ = [
    'Source',
    'JSONFileSource',
    'FolderSourceBase',
    'FileTypeFolderSourceBase',
    'JSONFolderSource',
]

import os
import json
import glob

try:
    import mixon
    HAS_MIXON = True
except ImportError:
    HAS_MIXON = False


class Source:
    def __iter__(self):
        return


class JSONFileSource(Source):
    def __init__(self, file_path):
        self.file_path = file_path

    def __iter__(self):
        with open(self.file_path) as f:
            data = json.load(f, strict=False)
            assert isinstance(data, list)
            return iter(data)


class FolderSourceBase(Source):
    def __init__(self, folder_path):
        self.folder_path = folder_path

    def __iter__(self):
        for file_path in self.list_files():
            yield self.load_file(file_path)

    def list_files(self):
        return glob.iglob(os.path.join(self.folder_path, '*'))

    def load_file(self, file_path):
        raise NotImplementedError


class FileTypeFolderSourceBase(FolderSourceBase):
    ext = ''

    def list_files(self):
        if not self.ext:
            file_pattern = '*'
        else:
            file_pattern = '*.{}'.format(self.ext.lstrip('.'))
        return glob.iglob(os.path.join(self.folder_path, file_pattern))


class JSONFolderSource(FileTypeFolderSourceBase):
    ext = 'json'

    def load_file(self, file_path):
        with open(file_path) as f:
            return json.load(f)


class MIXONFolderSource(FileTypeFolderSourceBase):
    ext = 'mixon'

    def load_file(self, file_path):
        if not HAS_MIXON:
            raise RuntimeError("Requires mixon! (pip install mixon)")

        with open(file_path) as f:
            return mixon.load(f)
